package com.tzld.piaoquan.featurestools.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.featurestools.model.bo.EmbeddingResult;
import com.tzld.piaoquan.featurestools.service.TextEmbeddingService;
import com.tzld.piaoquan.featurestools.util.HttpClientUtil;
import com.tzld.piaoquan.featurestools.util.HttpPoolClientUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TextEmbeddingServiceImpl implements TextEmbeddingService {

    private static final HttpPoolClientUtil httpPoolClientUtil = HttpClientUtil.create(5000, 50000, 10, 20, 3, 3000);

    private static final String EMBED_URL = "http://192.168.203.4:6060/embed";
    private final Set<String> stopwords = new HashSet<>();

    @PostConstruct
    public void init() throws IOException {
        // 创建 ClassPathResource 对象，指定资源文件路径
        ClassPathResource resource = new ClassPathResource("stopwords_cn.txt");
        // 获取文件输入流
        BufferedReader reader = new BufferedReader(new InputStreamReader(resource.getInputStream()));
        String line;
        while ((line = reader.readLine()) != null) {
            stopwords.add(line.trim());
        }
    }

    @Override
    public EmbeddingResult getEmbedding(String sampleText) {
        JSONObject result = sendEmbedRequest(sampleText);
        if (result == null) {
            return null;
        }
        JSONArray embeddings = result.getJSONArray("embeddings");
        List<String> words = result.getJSONArray("tokens").toJavaList(String.class);

        List<List<Double>> filteredEmbeddings = basicFilterEmbeddings(words, embeddings);
        List<String> filterWords = basicFilterWords(words);
        List<Double> doubleList = averageEmbeddings(filteredEmbeddings);
        String embeddingStr = doubleList.stream()
                .map(String::valueOf)
                .collect(Collectors.joining("|"));
        String wordStr = String.join(",", filterWords);

        EmbeddingResult embeddingResult = new EmbeddingResult();
        embeddingResult.setEmbeddingRes(embeddingStr);
        embeddingResult.setWords(wordStr);
        return embeddingResult;
    }







    public List<List<Double>> basicFilterEmbeddings(List<String> words, JSONArray embeddingsArray) {
        List<List<Double>> saveEmbeddings = new ArrayList<>();
        for (int i = 0; i < embeddingsArray.size(); i++) {
            List<Double> embeddings = embeddingsArray.getJSONArray(i).toJavaList(Double.class);
            String word = words.get(i);
            if (!stopwords.contains(word)) {
                saveEmbeddings.add(embeddings);
            }
        }
        return saveEmbeddings;
    }

    public List<String> basicFilterWords(List<String> words) {
        List<String> saveWord = new ArrayList<>();
        for (String word : words) {
            if (!stopwords.contains(word)) {
                saveWord.add(word);
            }
        }
        return saveWord;
    }

    public List<Double> averageEmbeddings(List<List<Double>> embeddings) {
        if (embeddings.isEmpty()) {
            return new ArrayList<>();
        }
        int length = embeddings.get(0).size();
        double[] sums = new double[length];
        for (List<Double> embedding : embeddings) {
            for (int i = 0; i < length; i++) {
                sums[i] += embedding.get(i);
            }
        }
        int numArrays = embeddings.size();
        List<Double> averages = new ArrayList<>();
        for (double sum : sums) {
            averages.add(sum / numArrays);
        }
        return averages;
    }

    public JSONObject sendEmbedRequest(String textContent) {
        try {
            JSONObject param = new JSONObject();
            param.put("text", textContent);
            String post = httpPoolClientUtil.post(EMBED_URL, param.toJSONString());
            if (StringUtils.isNotEmpty(post)) {
                return JSONObject.parseObject(post);
            }
        } catch (Exception e) {
            log.error("sendEmbedRequest error", e);
        }
        return null;
    }


}
